@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.79 1999/10/11 20:53:31 nisse Exp $ */
+ * $Id: client.c,v 1.80 1999/11/03 22:13:52 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -410,6 +410,11 @@ do_client_io(struct command *s UNUSED,
   channel->send = do_send;
 
   session->in->super.read = make_channel_read_data(channel);
+
+  /* FIXME: Perhaps there is some way to arrange that channel.c calls
+   * the CHANNEL_SEND method instead? */
+  if (session->super.send_window_size)
+    session->in->super.want_read = 1;
   
   session->in->super.close_callback
     = make_channel_read_close_callback(channel);
