@@ -1,6 +1,6 @@
 /* connection_commands.c
  *
- * $Id: connection_commands.c,v 1.15 1999/10/24 20:12:31 nisse Exp $
+ * $Id: connection_commands.c,v 1.16 1999/11/03 22:13:55 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -171,7 +171,7 @@ do_line(struct line_handler **h,
 #endif /* WITH_SSH1_FALLBACK */
 
 #if DATAFELLOWS_WORKAROUNDS
-	    if ( (swver_len > 6) && !memcmp(swver, "2.0.", 4)
+	    if ( (swver_len >= 6) && !memcmp(swver, "2.0.", 4)
 		 /* FIXME: Perhaps do a numerical comparison here? */
 		 && (memcmp(swver + 4, "13", 2) <= 0) )
 	      {
