@@ -1,6 +1,6 @@
 /* exception.h
  *
- * $Id: exception.h,v 1.11 1999/10/11 20:55:48 nisse Exp $ */
+ * $Id: exception.h,v 1.13 1999/11/07 21:01:16 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -53,10 +53,11 @@
 */
 
 void exception_raise(struct exception_handler *e,
-		     const struct exception *h);
+		     const struct exception *h,
+		     const char *context);
 
 #if DEBUG_TRACE
-#  define EXCEPTION_RAISE exception_raise
+#  define EXCEPTION_RAISE(h, e) exception_raise((h), (e), HANDLER_CONTEXT)
 #else /* !DEBUG_TRACE */
 #  define EXCEPTION_RAISE(h, e)  ((h)->raise((h), (e)))
 #endif /* !DEBUG_TRACE */
@@ -86,7 +87,8 @@ void exception_raise(struct exception_handler *e,
 #define EXC_IO_READ 0x2004
 #define EXC_IO_WRITE 0x2005
 #define EXC_IO_OPEN_WRITE 0x2006
-#define EXC_IO_LISTEN 0x2007
+#define EXC_IO_OPEN_READ 0x2007
+#define EXC_IO_LISTEN 0x2008
 
 /* Authorization errors */
 #define EXC_AUTH 0x4000
