@@ -2,7 +2,7 @@
  *
  * Create a packet from a format string and arguments.
  *
- * $Id: format.h,v 1.25 1999/11/16 21:15:44 nisse Exp $ */
+ * $Id: format.h,v 1.26 1999/11/21 22:36:04 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -97,6 +97,9 @@ void format_decimal(unsigned length, UINT8 *buffer, UINT32 n);
 int lsh_string_eq(const struct lsh_string *a, const struct lsh_string *b);
 int lsh_string_eq_l(const struct lsh_string *a, UINT32 length, const UINT8 *b);
 
+int lsh_string_prefixp(const struct lsh_string *prefix,
+		       const struct lsh_string *s);
+
 #if 0
 int lsh_string_cmp(const struct lsh_string *a, const struct lsh_string *b);
 int lsh_string_cmp_l(const struct lsh_string *a, UINT32 length, const UINT8 *b);
