@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: io.h,v 1.49 1999/11/16 21:20:46 nisse Exp $ */
+ * $Id: io.h,v 1.50 1999/11/28 18:15:56 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -36,9 +36,6 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 
-/* Forward declarations */
-
-struct lsh_fd;
 
 #define GABA_DECLARE
 #include "io.h.x"
