@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.83 1999/11/16 21:25:58 nisse Exp $ */
+ * $Id: lsh.c,v 1.87 1999/11/29 21:19:01 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -31,6 +31,7 @@
 #include "client.h"
 #include "client_keyexchange.h"
 #include "client_pty.h"
+#include "client_userauth.h"
 #include "compress.h"
 #include "connection_commands.h"
 #include "crypto.h"
@@ -41,14 +42,15 @@
 #include "lookup_verifier.h"
 #include "randomness.h"
 #include "service.h"
-#include "ssh.h"
+#include "sexp.h"
 #include "spki.h"
+#include "ssh.h"
 #include "tcpforward_commands.h"
 #include "tty.h"
-#include "userauth.h"
 #include "werror.h"
 #include "xalloc.h"
 
+#include <assert.h>
 #include <errno.h>
 #include <locale.h>
 #include <stdio.h>
@@ -77,70 +79,159 @@
 
 /* GABA:
    (class
-     (name sloppy_host_db)
+     (name client_host_db)
      (super lookup_verifier)
      (vars
+       (db object spki_context)
+       (access object sexp)
+       (host object address_info)
+       ; Allow unauthorized keys
+       (sloppy . int)
+       ; If non-null, append an ACL for the received key to this file.
+       (file object abstract_write)
        (hash object hash_algorithm) ; For fingerprinting
-       (algorithm object signature_algorithm)))
+       ;; (algorithm object signature_algorithm)
+       ))
 */
 
 static struct verifier *
-do_sloppy_lookup(struct lookup_verifier *c,
+do_client_lookup(struct lookup_verifier *c,
 		 int method,
 		 struct lsh_string *keyholder UNUSED,
 		 struct lsh_string *key)
 {
-  CAST(sloppy_host_db, self, c);
+  CAST(client_host_db, self, c);
+  struct spki_subject *subject;
 
-  if (method != ATOM_SSH_DSS)
-    return NULL;
-  
-  if (!quiet_flag)
+  switch (method)
     {
-      /* Display fingerprint */
-      struct sexp *e;
-      struct lsh_string *canonical;
-      struct hash_instance *hash;
-      struct lsh_string *digest;
-      
-      e = keyblob2spki(key);
+    case ATOM_SSH_DSS:
+      {
+	struct dsa_verifier *v = make_ssh_dss_verifier(key->length, key->data);
+	if (!v)
+	  {
+	    werror("do_client_lookup: Invalid ssh-dss key.\n");
+	    return NULL;
+	  }
+	subject = SPKI_LOOKUP(self->db,
+			      dsa_to_spki_public_key(&v->public),
+			      &v->super);
+	assert(subject);
+	assert(subject->verifier);
+	break;
+      }
+    case ATOM_SPKI:
+      {
+	struct sexp *e = string_to_sexp(key, 0);
+	if (!e)
+	  {
+	    werror("do_client_lookup: Invalid spki s-expression.\n");
+	    return NULL;
+	  }
+	  
+	subject = SPKI_LOOKUP(self->db, e, NULL);
+	if (!subject)
+	  {
+	    werror("do_client_lookup: Invalid spki key.\n");
+	    return NULL;
+	  }
+	if (!subject->verifier)
+	  {
+	    werror("do_client_lookup: Valid SPKI subject, but no key available.\n");
+	    return NULL;
+	  }
+	break;
+      }
+    default:
+      werror("do_client_lookup: Unknown key type. Should not happen!\n");
+      return NULL;
+    }
 
-      if (!e)
+  assert(subject->key);
+  
+  /* Check authorization */
+
+  if (SPKI_AUTHORIZE(self->db, subject, self->access))
+    {
+      verbose("SPKI authorization successful!");
+    }
+  else
+    {
+      verbose("SPKI authorization failed.\n");
+      if (!self->sloppy)
 	{
-	  werror("Invalid host key.\n");
+	  werror("lsh: Server's hostkey is not trusted. Disconnecting.\n");
 	  return NULL;
 	}
       
-      canonical = SEXP_FORMAT(e, SEXP_CANONICAL, 0);
-      hash = MAKE_HASH(self->hash);
-      digest = lsh_string_alloc(hash->hash_size);
-      
-      HASH_UPDATE(hash, canonical->length, canonical->data);
-      HASH_DIGEST(hash, digest->data);
-      KILL(hash);
-
-      if (!yes_or_no(ssh_format("Received unauthenticated key for host FOO\n"
-				"Fingerprint: %lxS\n"
-				"Do you trust this key? (y/n) ",
-				/* keyholder, */ digest), 0, 1))
+      /* Ok, let's see if we want to use this untrusted key. */
+      if (!quiet_flag)
 	{
-	  lsh_string_free(canonical);
-	  lsh_string_free(digest);
-	  return NULL;
+	  /* Display fingerprint */
+	  struct lsh_string *fingerprint
+	    = hash_string(self->hash,
+			  sexp_format(subject->key, SEXP_CANONICAL, 0),
+			  1);
+			  
+	  if (!yes_or_no(ssh_format("Received unauthenticated key for host %S\n"
+				    "Fingerprint: %lfxS\n"
+				    "Do you trust this key? (y/n) ",
+				    self->host->ip, fingerprint), 0, 1))
+	    return NULL;
+	}
+      
+      if (self->file)
+	{
+#if 0
+	  struct io_fd *fd = io_write_file(self->backend, self->filename,
+					   O_CREAT | O_APPEND | O_WRONLY,
+					   0600, 500, NULL,
+					   make_report_exception_handler(EXC_IO, EXC_IO,
+									 "Writing new ACL: "
+									 &default_exception_handler,
+									 HANDLER_CONTEXT));
+	  if (fd)
+	    {
+#endif
+	      A_WRITE(self->file, ssh_format("\n; ACL for host %lS\n", self->host->ip));
+	      A_WRITE(self->file,
+		      sexp_format(sexp_l(2, sexp_a(ATOM_ACL),
+					 sexp_l(3, sexp_a(ATOM_ENTRY),
+						subject->key,
+						sexp_l(2, sexp_a(ATOM_TAG),
+						       self->access,
+						       -1),
+						-1),
+					 -1),
+				  SEXP_TRANSPORT, 0));
+	      A_WRITE(self->file, ssh_format("\n"));
+#if 0
+	      close_fd_nicely(&fd->super, 0);
+	    }
+	  else
+	    werror("Failed to open '%z' (errno = %i): %z.\n",
+		   self->filename, errno, STRERROR(errno));
+#endif
 	}
-      /* Save key */
     }
   
-  return MAKE_VERIFIER(self->algorithm, key->length, key->data);
+  return subject->verifier;
 }
 
 static struct lookup_verifier *
-make_sloppy_host_db(struct signature_algorithm *a)
+make_client_host_db(struct spki_context *db,
+		    struct address_info *host,
+		    int sloppy,
+		    struct abstract_write *file)
 {
-  NEW(sloppy_host_db, res);
+  NEW(client_host_db, res);
 
-  res->super.lookup = do_sloppy_lookup;
-  res->algorithm = a;
+  res->super.lookup = do_client_lookup;
+  res->db = db;
+  res->access = make_ssh_hostkey_tag(host);
+  res->host = host;
+  res->sloppy = sloppy;
+  res->file = file;
   res->hash = &sha1_algorithm;
 
   return &res->super;
@@ -256,9 +347,14 @@ static int remember_tty(int fd)
 
 /* Option parsing */
 
-#define ARG_NOT 0x200
+#define ARG_NOT 0x400
+
 #define OPT_NO_PUBLICKEY 0x201
 
+#define OPT_SLOPPY 0x202
+#define OPT_STRICT 0x203
+#define OPT_CAPTURE 0x204
+
 static const struct argp_option
 main_options[] =
 {
@@ -268,6 +364,13 @@ main_options[] =
   { "identity", 'i',  "Identity key", 0, "Use this key to authenticate.", 0 },
   { "no-publickey", OPT_NO_PUBLICKEY, NULL, 0,
     "Don't try publickey user authentication.", 0 },
+  { "sloppy-host-authentication", OPT_SLOPPY, NULL, 0,
+    "Allow untrusted hostkeys.", 0 },
+  { "strict-host-authentication", OPT_STRICT, NULL, 0,
+    "Never, never, ever trust an unknown hostkey. (default)", 0 },
+  { "capture-to", OPT_CAPTURE, "File", 0,
+    "When a new hostkey is received, append an ACL expressing trust in the key. "
+    "In sloppy mode, the default is ~/.lsh/captured_keys.", 0 },
   { NULL, 0, NULL, 0, "Actions:", 0 },
   { "forward-local-port", 'L', "local-port:target-host:target-port", 0, "", 0 },
   { "forward-remote-port", 'R', "remote-port:target-host:target-port", 0, "", 0 },
@@ -307,6 +410,11 @@ main_options[] =
        (user . "char *")
        (identities struct object_queue)
        (publickey . int)
+
+       (sloppy . int)
+       (capture . "const char *")
+       (capture_file object abstract_write)
+       
        ; -1 means default behaviour
        (with_pty . int)
 
@@ -337,6 +445,10 @@ make_options(struct alist *algorithms, struct io_backend *backend,
   self->user = getenv("LOGNAME");
   self->port = "ssh";
 
+  self->sloppy = 0;
+  self->capture = NULL;
+  self->capture_file = NULL;
+  
   self->with_pty = -1;
   self->start_shell = 1;
   self->with_remote_peers = 0;
@@ -388,112 +500,149 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 
       break;
     case ARGP_KEY_ARGS:
-      argp_error(state, "Providing a remote command on the command is not supported yet.");
+      argp_error(state, "Providing a remote command on the command line is not supported yet.");
       break;
 
     case ARGP_KEY_END:
-      if (!self->user)
-	{
-	  argp_error(state, "No user name given. Use the -l option, or set LOGNAME in the environment.");
-	  break;
-	}
+      {
+	char *home = getenv("HOME");
+	if (!self->user)
+	  {
+	    argp_error(state, "No user name given. Use the -l option, or set LOGNAME in the environment.");
+	    break;
+	  }
 
-#if WITH_TCP_FORWARD
-      if (self->remote_forward)
-	object_queue_add_tail
-	  (&self->actions,
-	   &make_install_fix_channel_open_handler
-	   (ATOM_FORWARDED_TCPIP, &channel_open_forwarded_tcpip)->super);
-#endif /* WITH_TCP_FORWARD */
-      
-      /* Add shell action */
-      if (self->start_shell)
 	{
-	  int in;
-	  int out;
-	  int err;
-
-	  struct command *get_pty = NULL;
+	  struct lsh_string *tmp = NULL;
+	  const char *s = NULL;
 	  
-	  struct object_list *session_requests;
-      
-#if WITH_PTY_SUPPORT
-	  if (self->with_pty)
+	  if (self->capture)
+	    s = self->capture;
+	  else if (self->sloppy)
 	    {
-	      if (tty_fd < 0)
-		{
-		  werror("lsh: No tty available.\n");
-		}
+	      tmp = ssh_format("%lz/.lsh/captured_keys%c", home, 0);
+	      s = tmp->data;
+	    }
+	  if (s)
+	    {
+	      struct io_fd *f
+		= io_write_file(self->backend, s,
+				O_CREAT | O_APPEND | O_WRONLY,
+				0600, 500, NULL,
+				make_report_exception_handler(EXC_IO, EXC_IO,
+							      "Writing new ACL: ",
+							      &default_exception_handler,
+							      HANDLER_CONTEXT));
+	      if (f)
+		self->capture_file = &f->write_buffer->super;
 	      else
 		{
-		  if (! (remember_tty(tty_fd)
-			 && (get_pty = make_pty_request(tty_fd))))
-		    {
-		      werror("lsh: Can't use tty (probably getattr or atexit() failed.\n");
-		    }
+		  werror("Failed to open '%z' (errno = %i): %z.\n",
+			 s, errno, STRERROR(errno));
 		}
 	    }
-	  /* FIXME: We need a non-varargs constructor for lists. */
-	  if (get_pty)
-	    session_requests = make_object_list(2, get_pty, start_shell(), -1);
-	  else
-#endif /* WITH_PTY_SUPPORT */
-	    session_requests = make_object_list(1, start_shell(), -1);
-	  
-	  in = STDIN_FILENO;
-	  out = STDOUT_FILENO;
-	  
-	  if ( (err = dup(STDERR_FILENO)) < 0)
-	    {
-	      argp_failure(state, EXIT_FAILURE, errno, "Can't dup stderr: %s", STRERROR(errno));
-	      fatal("Can't happen.\n");
-	    }
-
-	  set_error_stream(STDERR_FILENO, 1);
-	  
-	  /* Exit code if no session is established */
-	  *self->exit_code = 17;
-	  
+	  lsh_string_free(tmp);
+	}
+	
+#if WITH_TCP_FORWARD
+	if (self->remote_forward)
 	  object_queue_add_tail
 	    (&self->actions,
-	     make_start_session
-	     (make_open_session_command(make_client_session
-					(io_read(make_io_fd(self->backend, in, self->handler),
-						 NULL, NULL),
-					 io_write(make_io_fd(self->backend, out, self->handler),
-						  BLOCK_SIZE, NULL),
-					 io_write(make_io_fd(self->backend, err, self->handler),
-						  BLOCK_SIZE, NULL),
-					 WINDOW_SIZE,
-					 self->exit_code)),
-	      session_requests));
-	}
-	  
-      if (object_queue_is_empty(&self->actions))
-	{
-	  argp_error(state, "No actions given.");
-	  break;
-	}
+	     &make_install_fix_channel_open_handler
+	     (ATOM_FORWARDED_TCPIP, &channel_open_forwarded_tcpip)->super);
+#endif /* WITH_TCP_FORWARD */
+      
+	/* Add shell action */
+	if (self->start_shell)
+	  {
+	    int in;
+	    int out;
+	    int err;
 
-      if (self->publickey && object_queue_is_empty(&self->identities))
-	{
-	  char *home = getenv("HOME");
-	  if (home)
-	    {
-	      struct lsh_string *idfile = 
-		ssh_format("%lz/.lsh/identity%c", home, 0);
-	      struct keypair *id;
-	      if ((id = read_spki_key_file(idfile->data, self->random, self->handler)))
-		{
-		  verbose("Adding identity %S\n", idfile);
-		  object_queue_add_tail(&self->identities, &id->super);
-		}
-	      lsh_string_free(idfile);
-	    }
-	}
+	    struct command *get_pty = NULL;
+	  
+	    struct object_list *session_requests;
       
-      break;
+#if WITH_PTY_SUPPORT
+	    if (self->with_pty)
+	      {
+		if (tty_fd < 0)
+		  {
+		    werror("lsh: No tty available.\n");
+		  }
+		else
+		  {
+		    if (! (remember_tty(tty_fd)
+			   && (get_pty = make_pty_request(tty_fd))))
+		      {
+			werror("lsh: Can't use tty (probably getattr or atexit() failed.\n");
+		      }
+		  }
+	      }
+	    /* FIXME: We need a non-varargs constructor for lists. */
+	    if (get_pty)
+	      session_requests = make_object_list(2, get_pty, start_shell(), -1);
+	    else
+#endif /* WITH_PTY_SUPPORT */
+	      session_requests = make_object_list(1, start_shell(), -1);
+	  
+	    in = STDIN_FILENO;
+	    out = STDOUT_FILENO;
+	  
+	    if ( (err = dup(STDERR_FILENO)) < 0)
+	      {
+		argp_failure(state, EXIT_FAILURE, errno, "Can't dup stderr: %s", STRERROR(errno));
+		fatal("Can't happen.\n");
+	      }
+
+	    set_error_stream(STDERR_FILENO, 1);
+	  
+	    /* Exit code if no session is established */
+	    *self->exit_code = 17;
+	  
+	    object_queue_add_tail
+	      (&self->actions,
+	       make_start_session
+	       (make_open_session_command(make_client_session
+					  (io_read(make_io_fd(self->backend, in, self->handler),
+						   NULL, NULL),
+					   io_write(make_io_fd(self->backend, out, self->handler),
+						    BLOCK_SIZE, NULL),
+					   io_write(make_io_fd(self->backend, err, self->handler),
+						    BLOCK_SIZE, NULL),
+					   WINDOW_SIZE,
+					   self->exit_code)),
+		session_requests));
+	  }
+	  
+	if (object_queue_is_empty(&self->actions))
+	  {
+	    argp_error(state, "No actions given.");
+	    break;
+	  }
+
+	if (self->publickey && object_queue_is_empty(&self->identities))
+	  {
+	    if (home)
+	      {
+		struct lsh_string *idfile = 
+		  ssh_format("%lz/.lsh/identity%c", home, 0);
+		struct keypair *id;
+		if ((id = read_spki_key_file(idfile->data,
+					     make_linear_alist(1, ATOM_DSA,
+							       make_dsa_algorithm(self->random),
+							       -1),
+					     self->handler)))
+		  {
+		    verbose("Adding identity %S\n", idfile);
+		    object_queue_add_tail(&self->identities, &id->super);
+		  }
+		lsh_string_free(idfile);
+	      }
+	  }
       
+	break;
+      }
     case 'p':
       self->port = arg;
       break;
@@ -503,7 +652,12 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
     case 'i':
       {
 	struct keypair *id;
-	if ((id = read_spki_key_file(arg, self->random, self->handler)))
+	/* FIXME: Get the algorithms alist from some parameter. */
+	if ((id = read_spki_key_file(arg,
+				     make_linear_alist(1, ATOM_DSA,
+						       make_dsa_algorithm(self->random),
+						       -1),
+				     self->handler)))
 	  {
 	    verbose("Adding identity %s\n", arg);
 	    object_queue_add_tail(&self->identities, &id->super);
@@ -513,6 +667,18 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
     case OPT_NO_PUBLICKEY:
       self->publickey = 0;
       break;
+
+    case OPT_SLOPPY:
+      self->sloppy = 1;
+      break;
+
+    case OPT_STRICT:
+      self->sloppy = 0;
+      break;
+
+    case OPT_CAPTURE:
+      self->capture = arg;
+      break;
       
     case 'L':
       {
@@ -676,11 +842,12 @@ int main(int argc, char **argv)
 
   struct randomness *r;
   struct diffie_hellman_method *dh;
-  struct keyexchange_algorithm *kex;
   struct alist *algorithms;
   struct make_kexinit *make_kexinit;
+  struct lookup_verifier *lookup;
+  
   struct alist *lookup_table; /* Alist of signature-algorithm -> lookup_verifier */
-
+  
   /* int in, out, err; */
 
   /* FIXME: A single exception handler everywhere seems a little to
@@ -706,22 +873,33 @@ int main(int argc, char **argv)
   r = make_reasonably_random();
   dh = make_dh1(r);
 
-  /* No randomness is needed for verifying signatures */
-  lookup_table = make_alist(1,
-			    ATOM_SSH_DSS, make_sloppy_host_db(make_dsa_algorithm(NULL)),
-			    -1); 
-
-  kex = make_dh_client(dh, lookup_table);
-  algorithms = many_algorithms(2, 
-			       ATOM_DIFFIE_HELLMAN_GROUP1_SHA1, kex,
+  /* FIXME: Is this really used? */
+  algorithms = many_algorithms(1,
 			       ATOM_SSH_DSS, make_dsa_algorithm(r),
 			       -1);
 
-
   options = make_options(algorithms, backend, r, handler, &lsh_exit_code);
 
   argp_parse(&main_argp, argc, argv, ARGP_IN_ORDER, NULL, options);
   
+  /* No randomness is needed for verifying signatures */
+  lookup = make_client_host_db(make_spki_context
+			       (make_alist(1,
+					   ATOM_DSA,
+					   make_dsa_algorithm(NULL), -1)),
+			       options->remote,
+			       options->sloppy,
+			       options->capture_file);
+  
+  lookup_table = make_alist(2,
+			    ATOM_SSH_DSS, lookup,
+			    ATOM_SPKI, lookup,
+			    -1); 
+
+  ALIST_SET(algorithms,
+	    ATOM_DIFFIE_HELLMAN_GROUP1_SHA1,
+	    make_dh_client(dh, lookup_table));
+  
   make_kexinit
     = make_simple_kexinit(r,
 			  make_int_list(1, ATOM_DIFFIE_HELLMAN_GROUP1_SHA1,
