@@ -2,7 +2,7 @@
  *
  * main server program.
  *
- * $Id: lshd.c,v 1.73 1999/11/07 20:17:51 nisse Exp $ */
+ * $Id: lshd.c,v 1.76 1999/11/29 21:22:02 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -44,7 +44,6 @@
 #include "server_keyexchange.h"
 #include "server_pty.h"
 #include "server_session.h"
-#include "server_userauth.h"
 #include "sexp.h"
 #include "sexp_commands.h"
 #include "spki.h"
@@ -52,7 +51,7 @@
 #include "tcpforward.h"
 #include "tcpforward_commands.h"
 #include "tcpforward_commands.h"
-#include "userauth.h"
+#include "server_userauth.h"
 #include "werror.h"
 #include "xalloc.h"
 
@@ -381,7 +380,9 @@ int main(int argc, char **argv)
       
   /* Read the hostkey */
   keys = make_alist(0, -1);
-  if (!(hostkey = read_spki_key_file(options->hostkey, r, &ignore_exception_handler)))
+  if (!(hostkey = read_spki_key_file(options->hostkey,
+				     make_alist(1, ATOM_DSA, make_dsa_algorithm(r), -1),
+				     &ignore_exception_handler)))
     {
       werror("lshd: Could not read hostkey.\n");
       return EXIT_FAILURE;
@@ -399,7 +400,7 @@ int main(int argc, char **argv)
   authorization_lookup
     = make_alist(1,
 		 ATOM_SSH_DSS, make_authorization_db(ssh_format("authorized_keys_sha1"),
-						     make_dsa_algorithm(NULL),
+						     /* make_dsa_algorithm(NULL), */
 						     &sha1_algorithm),
 		 
 		 -1);
@@ -465,8 +466,7 @@ int main(int argc, char **argv)
 					, ATOM_SHELL,
 					make_shell_handler(backend,
 							   reaper),
-					-1),
-				       backend),
+					-1)),
 				      connection_hooks),
 				     -1))),
 	   -1)));
