@@ -2,7 +2,7 @@
  *
  * An implementation of Ron Rivest's S-expressions, used in spki.
  *
- * $Id: sexp.c,v 1.19 1999/11/16 21:41:41 nisse Exp $ */
+ * $Id: sexp.c,v 1.23 1999/11/28 23:20:05 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -738,7 +738,19 @@ int sexp_eq(struct sexp *e, UINT32 length, const UINT8 *name)
 
   return c && lsh_string_eq_l(c, length, name);
 }
-  
+
+/* Assumes that both expressions are atoms */
+int sexp_atom_eq(struct sexp *a, struct sexp *b)
+{
+  struct lsh_string *ac = sexp_contents(a);
+  struct lsh_string *ad = sexp_display(a);
+  struct lsh_string *bc = sexp_contents(b);
+  struct lsh_string *bd = sexp_display(b);
+
+  return lsh_string_eq(ac, bc)
+    && (ad ? (bd && lsh_string_eq(ad, bd))
+	: !bd);
+}
 
 #if 0
 int sexp_eqz(const struct sexp *e, const char *s)
@@ -806,16 +818,19 @@ struct sexp *sexp_assz(struct sexp_iterator *i, const char *name)
 struct sexp *sexp_assq(struct sexp_iterator *i, int atom)
 {
   struct sexp_iterator *inner;
-  if (SEXP_ASSOC(i, get_atom_length(atom), get_atom_name(atom), &inner))
+  if (SEXP_ASSOC(i, get_atom_length(atom), get_atom_name(atom), &inner)
+      && (SEXP_LEFT(inner) == 1))
     {
       struct sexp *value = SEXP_GET(inner);
+      assert(value);
+      
       KILL(inner);
       return value;
     }
   else
     return NULL;
 }
-     
+
 int sexp_get_un(struct sexp_iterator *i, int atom, mpz_t n)
 {
   struct lsh_string *s = sexp2string(sexp_assq(i, atom));
