@@ -2,7 +2,7 @@
  *
  * An implementation of Ron Rivest's S-expressions, used in spki.
  *
- * $Id: sexp.h,v 1.22 1999/11/16 21:41:42 nisse Exp $ */
+ * $Id: sexp.h,v 1.24 1999/11/28 18:35:23 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -34,8 +34,6 @@
 
 #include "sexp_table.h"
 
-/* Forward declaration */
-struct sexp_iterator;
 
 #define GABA_DECLARE
 #include "sexp.h.x"
@@ -93,6 +91,7 @@ struct sexp_iterator;
 #define SEXP_GET(i) ((i)->get((i)))
 #define SEXP_SET(i, v) ((i)->set((i), (v)))
 #define SEXP_ASSOC(s, l, n, i) ((s)->assoc((s), (l), (n), (i)))
+#define SEXP_LEFT(i) ((i)->left((i)))
 #define SEXP_NEXT(i) ((i)->next((i)))
 
 /* Syntax styles */
@@ -112,6 +111,7 @@ struct lsh_string *encode_base64(struct lsh_string *s,
 
 /* Creating sexps */
 
+/* FIXME: Rename this to sexp_s(d, c)? */
 /* Consumes its args (display may be NULL) */
 struct sexp *make_sexp_string(struct lsh_string *d, struct lsh_string *c);
 
@@ -166,6 +166,7 @@ int sexp_bignum_s(const struct sexp *e, mpz_t n);
 
 /* Utility functions for parsing spki objects. */
 
+int sexp_atom_eq(struct sexp *a, struct sexp *b);
 int sexp_eq(struct sexp *e, UINT32 length, const UINT8 *name);
 struct sexp *sexp_assq(struct sexp_iterator *i, int atom);
 
