@@ -1,11 +1,11 @@
 /* tcpforward.c
  *
- * $Id: tcpforward.c,v 1.32 1999/10/24 20:33:26 nisse Exp $
+ * $Id: tcpforward.c,v 1.34 1999/11/16 22:13:45 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
  *
- * Copyright (C) 1998 Balazs Scheidler, Niels Mller
+ * Copyright (C) 1998 Balzs Scheidler, Niels Mller
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -74,7 +74,7 @@ lookup_forward(struct object_queue *q,
       CAST(forwarded_port, f, n);
       
       if ( (port == f->listen->port)
-	   && (lsh_string_cmp_l(f->listen->ip, length, ip) == 0) )
+	   && lsh_string_eq_l(f->listen->ip, length, ip) )
 	return f;
     }
   return NULL;
@@ -89,7 +89,7 @@ remove_forward(struct object_queue *q, int null_ok,
       CAST(local_port, f, n);
       
       if ( (port == f->super.listen->port)
-	   && (lsh_string_cmp_l(f->super.listen->ip, length, ip) == 0) )
+	   && lsh_string_eq_l(f->super.listen->ip, length, ip) )
 	{
 	  if (null_ok || f->socket)
 	    {
